package ch.fritteli.maze.server;

import ch.fritteli.maze.server.handler.CreateHandler;
import ch.fritteli.maze.server.handler.RenderV1Handler;
import ch.fritteli.maze.server.handler.RenderV2Handler;
import io.undertow.Undertow;
import io.undertow.server.RoutingHandler;
import io.vavr.control.Try;
import java.net.InetSocketAddress;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class MazeServer {

    @NonNull
    private final Undertow undertow;

    private MazeServer(@NonNull final ServerConfig config) {
        final String hostAddress = config.getAddress().getHostAddress();
        final int port = config.getPort();
        log.info("Starting Server at http://{}:{}/", hostAddress, port);
        final RoutingHandler routingHandler = new RoutingHandler()
                .get(CreateHandler.PATH_TEMPLATE, new CreateHandler())
                .post(RenderV1Handler.PATH_TEMPLATE, new RenderV1Handler())
                .post(RenderV2Handler.PATH_TEMPLATE, new RenderV2Handler());

        this.undertow = Undertow.builder()
                .addHttpListener(port, hostAddress)
                .setHandler(routingHandler)
                .build();
    }

    @NonNull
    public static Try<MazeServer> createAndStartServer() {
        return Try.of(ServerConfig::init)
                .flatMapTry(MazeServer::createAndStartServer);
    }

    @NonNull
    public static Try<MazeServer> createAndStartServer(@NonNull final ServerConfig config) {
        return Try.of(() -> new MazeServer(config))
                .peek(MazeServer::start);
    }

    private void start() {
        Runtime.getRuntime().addShutdownHook(new Thread(this::stop, "listener-stopper"));
        this.undertow.start();
        final InetSocketAddress address = (InetSocketAddress) this.undertow.getListenerInfo().get(0).getAddress();
        final String hostAddress = address.getAddress().getHostAddress();
        final int port = address.getPort();

        log.info("Listening on http://{}:{}", hostAddress, port);
    }

    private void stop() {
        log.info("Stopping server ...");
        this.undertow.stop();
        log.info("Server stopped.");
    }
}
